/*
 * TasksJDBMImpl.java
 *
 */

package com.builder.uk.todotasks;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import jdbm.RecordManager;
import jdbm.RecordManagerFactory;
import jdbm.btree.BTree;
import jdbm.helper.LongComparator;
import jdbm.helper.Tuple;
import jdbm.helper.TupleBrowser;

/**
 * JDBM implementation of Tasks
 */
public class TasksJDBMImpl implements Tasks {
    
    RecordManager recman;
    BTree tasktable;
       
    /**
     * Constructor for TasksJDBMImpl
     * @throws java.io.IOException thrown if an IO error occurs
     */
    public TasksJDBMImpl() throws IOException {
        recman=RecordManagerFactory.createRecordManager("tasks");
        
        long recid=recman.getNamedObject("tasktable");
        if(recid==0) {
            // There is no tasktable; we create it here
            tasktable=BTree.createInstance(recman,new LongComparator());
            recman.setNamedObject("tasktable",tasktable.getRecid());
            recman.commit();
        } else {
            tasktable=BTree.load(recman,recid);
        }
    }
    
    private Long generateTaskId() throws IOException {
        
        Long generatedTaskId;
        long recid=recman.getNamedObject("nextid");
        
        if(recid==0) {
             // There is no existing nextid; we create it
            generatedTaskId=new Long(1000L);
            recid=recman.insert(generatedTaskId);
            recman.setNamedObject("nextid",recid);
        } else {
            // We have an existing nextid; get the record
            generatedTaskId=(Long)recman.fetch(recid);
        }
        
        Long genid=generatedTaskId;
        generatedTaskId++;
        
        recman.update(recid,generatedTaskId);
        recman.commit();
        return genid;
    }
    
    public List<Long> getTaskIds() throws IOException {
        TupleBrowser tupleBrowser=tasktable.browse();
   
        ArrayList<Long> results=new ArrayList<Long>();
        Tuple tuple=new Tuple();
        
        while(tupleBrowser.getNext(tuple)) {
            Long key=(Long)tuple.getKey();
            results.add(key);
        }
        
        return results;
    }
    
    
    public Task createTask() throws IOException {
        Task toDoTask=new Task();
        Long l=generateTaskId();
        toDoTask.setTaskId(l);
        toDoTask.setPriority(Priority.NONE);
        toDoTask.setName("New Task");
        toDoTask.setDonePercent(0);
        tasktable.insert(l,toDoTask,false);
        recman.commit();
        fireTasksListenerTasksChanged(new TasksEvent(TasksEvent.TasksEventType.NEW,toDoTask));
        return toDoTask;
    }
    
    public void removeTask(Long taskId) throws IOException {
        tasktable.remove(taskId);
        recman.commit();
        fireTasksListenerTasksChanged(new TasksEvent(TasksEvent.TasksEventType.DELETED,taskId));
    }
    
    public int getTaskCount() {
        return tasktable.size();
    }
    
    public Task getTask(Long id) throws IOException {
        return (Task)tasktable.find(id);
    }
    
    public void update(Task t) throws IOException {
        Object o=tasktable.insert(t.getTaskId(),t,true);
        recman.commit();
        if(o==null)
        {
            fireTasksListenerTasksChanged(new TasksEvent(TasksEvent.TasksEventType.NEW,t));
        }
        else
        {
            fireTasksListenerTasksChanged(new TasksEvent(TasksEvent.TasksEventType.CHANGED,t));
        }
    }
    
    /**
     * Utility field used by event firing mechanism.
     */
    private javax.swing.event.EventListenerList listenerList =  null;
    
    /**
     * Registers TasksListener to receive events.
     * @param listener The listener to register.
     */
    public synchronized void addTasksListener(com.builder.uk.todotasks.TasksListener listener) {
        if (listenerList == null ) {
            listenerList = new javax.swing.event.EventListenerList();
        }
        listenerList.add(com.builder.uk.todotasks.TasksListener.class, listener);
    }
    
    /**
     * Removes TasksListener from the list of listeners.
     * @param listener The listener to remove.
     */
    public synchronized void removeTasksListener(com.builder.uk.todotasks.TasksListener listener) {
        listenerList.remove(com.builder.uk.todotasks.TasksListener.class, listener);
    }
    
    /**
     * Notifies all registered listeners about the event.
     *
     * @param event The event to be fired
     */
    private void fireTasksListenerTasksChanged(com.builder.uk.todotasks.TasksEvent event) {
        if (listenerList == null) return;
        Object[] listeners = listenerList.getListenerList();
        for (int i = listeners.length - 2; i >= 0; i -= 2) {
            if (listeners[i]==com.builder.uk.todotasks.TasksListener.class) {
                ((com.builder.uk.todotasks.TasksListener)listeners[i+1]).tasksChanged(event);
            }
        }
    }

    public void close() throws IOException {
        recman.close();
    }

    public List<Task> getTasks() throws IOException {
         TupleBrowser tupleBrowser=tasktable.browse();
   
        ArrayList<Task> results=new ArrayList<Task>();
        Tuple tuple=new Tuple();
        
        while(tupleBrowser.getNext(tuple)) {
            Task task=(Task)tuple.getValue();
            results.add(task);
        }
        
        return results;
    }
}
