/*
 * TasksModel.java
 *
 */

package com.builder.uk.todotasks;

import java.io.IOException;
import java.util.Date;
import java.util.LinkedList;
import java.util.List;
import javax.swing.event.TableModelEvent;
import javax.swing.table.TableModel;

/**
 * A TableModel which models the Task classes contained in the Tasks class
 */
public class TasksModel implements TableModel,TasksListener {
    
    private Tasks tasks;
    
    private List<Long> ordermap;
    
    /**
     * Constructor for TasksModel
     * @param tasks Tasks instance to be modelled
     * @throws java.io.IOException Thrown from errors in querying the Tasks class
     */
    public TasksModel(Tasks tasks) throws IOException {
        this.tasks=tasks;
        ordermap=new LinkedList<Long>(this.tasks.getTaskIds());
        this.tasks.addTasksListener(this);
    }
    
    /**
     * Gets the number of rows in the model
     * @return number of tasks
     */
    public int getRowCount() {
        return ordermap.size();
    }
    
    /**
     * Gets the number of columns in the model
     * @return number of columns
     */
    public int getColumnCount() {
        return 6;
    }
    
    /**
     * Gets column name
     * @param columnIndex column index
     * @return name of column at that index
     */
    public String getColumnName(int columnIndex) {
        switch(columnIndex) {
            case 0: return "Id";
            case 1: return "Name";
            case 2: return "Priority";
            case 3: return "Done";
            case 4: return "Completed";
            case 5: return "Due";
        }
        return "Unknown";
    }
    
    /**
     * Gets class of column at index
     * @param columnIndex column index
     * @return Class of column
     */
    public Class<?> getColumnClass(int columnIndex) {
        switch(columnIndex) {
            case 0: return Long.class;
            case 1: return String.class;
            case 2: return Priority.class;
            case 3: return Integer.class;
            case 4: return Date.class;
            case 5: return Date.class;
        }
        
        return String.class;
    }
    
    /**
     * test for cell editable status
     * @param rowIndex row index
     * @param columnIndex column index
     * @return true if the cell can be edited
     */
    public boolean isCellEditable(int rowIndex, int columnIndex) {
        if(columnIndex==0 || columnIndex==4) return false;
        return true;
    }
    
    /**
     * gets value of a cell
     * @param rowIndex row index
     * @param columnIndex column index
     * @return value of the cell
     */
    public Object getValueAt(int rowIndex, int columnIndex) {
        Long id=ordermap.get(rowIndex);
        Task task;
        try {
            task = this.tasks.getTask(id);
        } catch (IOException ex) {
            return "Error";
        }
        
        switch(columnIndex) {
            case 0: return task.getTaskId();
            case 1: return task.getName();
            case 2: return task.getPriority();
            case 3: return task.getDonePercent();
            case 4: return task.getCompletedDate();
            case 5: return task.getDueDate();
        }
        
        return "***";
    }
    
    /**
     * sets value of a cell
     * @param aValue value to set the cell to
     * @param rowIndex row index
     * @param columnIndex column index
     */
    public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
        Long id=ordermap.get(rowIndex);
        Task task;
        try {
            task = this.tasks.getTask(id);
        } catch (IOException ex) {
            ex.printStackTrace();
            return;
        }
        
        boolean changed=false;
        
        switch(columnIndex) {
            case 1:
                String nameValue=(String)aValue;
                if(!task.getName().equals((nameValue))) {
                    task.setName((String)aValue);
                    changed=true;
                }
                break;
            case 2:
                Priority priorityValue=(Priority)aValue;
                if(!task.getPriority().equals(priorityValue)) {
                    task.setPriority(priorityValue);
                    changed=true;
                }
                break;
            case 3:
                int donePercentValue=(Integer)aValue;
                if(task.getDonePercent()!=donePercentValue) {
                    task.setDonePercent((Integer)aValue);
                    task.setCompletedDate(donePercentValue==100?new Date():null);
                    changed=true;
                }
                break;
            case 5:
                Date dueValue=(Date)aValue;
                Date oldDueValue=task.getDueDate();
                if(oldDueValue==null)
                {
                    if(dueValue!=null)
                    {
                        task.setDueDate(dueValue);
                        changed=true;
                    }
                }
                else
                {
                    if(!oldDueValue.equals(dueValue))
                    {
                        task.setDueDate(dueValue);
                        changed=true;
                    }
                }
                break;
               
        }
        
        try {
            if(changed) this.tasks.update(task);
        } catch (IOException ex) {
            ex.printStackTrace();
        }
    }
    
    /**
     * Utility field used by event firing mechanism.
     */
    private javax.swing.event.EventListenerList listenerList =  null;
    
    /**
     * Registers TableModelListener to receive events.
     * @param listener The listener to register.
     */
    public synchronized void addTableModelListener(javax.swing.event.TableModelListener listener) {
        if (listenerList == null ) {
            listenerList = new javax.swing.event.EventListenerList();
        }
        listenerList.add(javax.swing.event.TableModelListener.class, listener);
    }
    
    /**
     * Removes TableModelListener from the list of listeners.
     * @param listener The listener to remove.
     */
    public synchronized void removeTableModelListener(javax.swing.event.TableModelListener listener) {
        listenerList.remove(javax.swing.event.TableModelListener.class, listener);
    }
    
    /**
     * Notifies all registered listeners about the event.
     *
     * @param event The event to be fired
     */
    private void fireTableModelListenerTableChanged(javax.swing.event.TableModelEvent event) {
        if (listenerList == null) return;
        Object[] listeners = listenerList.getListenerList();
        for (int i = listeners.length - 2; i >= 0; i -= 2) {
            if (listeners[i]==javax.swing.event.TableModelListener.class) {
                ((javax.swing.event.TableModelListener)listeners[i+1]).tableChanged(event);
            }
        }
        
    }
    
    /**
     * Recieves TasksEvent changes from the Tasks class to update the model
     * @param tasksevent TasksEvent containing changes
     */
    public void tasksChanged(TasksEvent tasksevent) {
        int index;
        
        switch(tasksevent.taskEventType) {
            
            case NEW:
                ordermap.add(tasksevent.taskId);
                index=ordermap.indexOf(tasksevent.taskId);
                fireTableModelListenerTableChanged(new TableModelEvent(this,index,index,TableModelEvent.ALL_COLUMNS,TableModelEvent.INSERT));
                break;
            case DELETED:
                index=ordermap.indexOf(tasksevent.taskId);
                ordermap.remove(tasksevent.taskId);
                fireTableModelListenerTableChanged(new TableModelEvent(this,index,index,TableModelEvent.ALL_COLUMNS,TableModelEvent.DELETE));
                break;
            case CHANGED:
                index=ordermap.indexOf(tasksevent.taskId);
                fireTableModelListenerTableChanged(new TableModelEvent(this,index));
                break;
                
        }
    }
    
}



