/*
 * TasksSession.java
 */

package com.builder.uk.todotasks;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.List;
import org.directwebremoting.ScriptBuffer;
import org.directwebremoting.ScriptSession;
import org.directwebremoting.WebContext;
import org.directwebremoting.WebContextFactory;

public class TasksSession implements TasksListener {
    
    Tasks tasks;
    
    SortOrder sortingfield=new SortOrder(SortOrder.TaskField.TASKID,false);
    
    List<Long> tasklist;
    
    WebContext wctx;
    
    public TasksSession() throws IOException {
        tasks=TasksFactory.getTasks();
        tasks.addTasksListener(this);
        tasklist=tasks.getTaskIds();
        
        wctx = WebContextFactory.get();
        Collections.sort(tasklist,new TaskComparator());
    }
    
    public void bind() {
        wctx=WebContextFactory.get();
    }
    
    public Task getTask(Long taskId) {
        try {
            return tasks.getTask(taskId);
        } catch (IOException ex) {
            ex.printStackTrace();
            return null;
        }
    }
    
    public Task createTask() throws IOException {
        return tasks.createTask();
    }
    
    public boolean deleteTask(Long taskId) {
        try {
            tasks.removeTask(taskId);
        } catch (IOException ex) {
            return false;
        }
        
        return true;
    }
    
    public boolean updateTask(Task task) {
        try {
            tasks.update(task);
        } catch (IOException ex) {
            ex.printStackTrace();
            return false;
        }
        return true;
    }
    
    
    
    public Task[] getAllTasks() {
        ArrayList<Task> al=new ArrayList<Task>();
        for(Long l:tasklist) {
            al.add(getTask(l));
        }
        return (Task [])al.toArray(new Task[al.size()]);
    }


//
// setSortOrder now, more usefully returns the newly ordered
// list of tasks rather than expecting the client to make another 
// call just to retrieve them
//
   
    public Task[] setSortOrder(SortOrder tf) {
        sortingfield=tf;
        Collections.sort(tasklist,new TaskComparator());
        return getAllTasks();
    }
    
    public SortOrder getSortOrder() {
        wctx=WebContextFactory.get();
        return sortingfield;
    }
    
//
// tasksChanged has been modified
// note firstly that the TasksEvent now carries an id only for deleted tasks
// but for other events carries the id and the task data.
//
    
    public void tasksChanged(final TasksEvent tasksevent) {

//
// We get the script session from our web context
        ScriptSession ss=wctx.getScriptSession();
// and then create a script buffer, associated with the servlet context of
// the web context
        ScriptBuffer sb=new ScriptBuffer(wctx.getServletContext());
 
        switch(tasksevent.taskEventType) {
            
            case CHANGED:
//
// On a changed task we take a copy of the current task list
                ArrayList<Long> tmptasks=new ArrayList<Long>(tasklist);

// We then sort the tasklist
                Collections.sort(tasklist, new TaskComparator());
         
// And then scan through the list to see if the *order* of the list has 
// changed
                boolean orderchanged=false;
                for(int i=0;i<tasklist.size();i++) {
                    if(!tasklist.get(i).equals(tmptasks.get(i))) {
                        orderchanged=true;
                        break;
                    }
                }

                if(orderchanged) {
// If the order has changed, we call a new javascript function, remoteAllTasksChanged
// and give it a full list of all the task data.
                    sb.appendScript("remoteAllTasksChanged(").appendData(getAllTasks()).appendScript(");");
                    ss.addScript(sb);
                } else {
// otherwise we call remoteTaskChanged with the details of the updated task. This will just update
// the row in the table in the javascript						
                    sb.appendScript("remoteTaskChanged(").appendData(tasksevent.task).appendScript(");");
                    ss.addScript(sb);
                }
                break;
            case NEW:
                int index=Collections.binarySearch(tasklist,tasksevent.taskId,new TaskComparator());
                if(index<0) {
                    tasklist.add(-index-1,tasksevent.taskId);
// When a task has been added, 	we call remoteTaskAdded, with the task object and an index number to assist
// the javascript in inserting the row in the correct position.
                    sb.appendScript("remoteTaskAdded(").appendData(tasksevent.task).appendScript(",").appendData(-index-1).appendScript(");");
                    ss.addScript(sb);
                }
                break;
            case DELETED:
                tasklist.remove(tasksevent.taskId);
                sb.appendScript("remoteTaskDeleted(").appendData(tasksevent.taskId).appendScript(");");
                ss.addScript(sb);
                
                break;
        }
        
    }
    
    
    private class TaskComparator implements Comparator {
        public int compare(Object o1, Object o2) {
            Task t1,t2;
            try {
                t1 = tasks.getTask((Long) o1);
                t2=tasks.getTask((Long)o2);
            } catch (IOException ex) {
                return 0;
            }
             
            int result=0;
            
            switch(sortingfield.getField()) {
                case TASKID:
                    result= t1.getTaskId().compareTo(t2.getTaskId());
                    break;
                case NAME:
                    result= t1.getName().compareTo(t2.getName());
                    break;
                case PRIORITY:
                    result= t1.getPriority().compareTo(t2.getPriority());
                    break;
                case DONE:
                    result= ((Integer)t1.getDonePercent()).compareTo(t2.getDonePercent());
                    break;
                case COMPLETED:
                    Date d1=t1.getCompletedDate();
                    Date d2=t2.getCompletedDate();
                    if(d1==null && d2==null) { result= 0; break; }
                    if(d1==null) { result=-1; break; }
                    if(d2==null) { result=1; break; };
                    result=d1.before(d2) ? -1 : 1;
                    break;
                case DUE:
                    Date dd1=t1.getDueDate();
                    Date dd2=t2.getDueDate();
                    if(dd1==null && dd2==null) {result=0; break; }
                    if(dd1==null) { result=-1; break; }
                    if(dd2==null) { result=1; break; }
                    result=dd1.before(dd2) ? -1 : 1;
                    break;
            }
            
//
// The next lines have been added to allow sorting to fall back on the task id value for
// ordering when the two compared objects have the same value.
//

            if(result==0 && sortingfield.getField()!=SortOrder.TaskField.TASKID)
            {
                result= t1.getTaskId().compareTo(t2.getTaskId());
            }
            
            if(sortingfield.isDescending()) return -result;
            return result;
        }
        
    }
    
    
}
